<?php
/**
 * Created by PhpStorm.
 * User: robin
 * Date: 9/30/15
 * Time: 7:04 PM
 */

namespace Luticate\Auth\Business;

use Illuminate\Http\Request;
use Luticate\Auth\DBO\LuticatePermissions;
use Luticate\Auth\DBO\LuticateUsersDbo;
use Luticate\Utils\LuController;
use Luticate\Utils\LuRoute;

class LuticateBusiness
{
    const TOKEN_HEADER = "X-Authorization";

    /**
     * @var LuticateUsersDbo
     */
    private static $_currentUser;

    public static function getCurrentUser()
    {
        return self::$_currentUser;
    }

    /**
     * @param $permissions string[]
     * @param $request Request
     * @return bool
     */
    public static function authFilter($permissions, $request)
    {
        /**
         * @var $user LuticateUsersDbo
         */
        $user = null;
        $token = $request->header(self::TOKEN_HEADER);
        if ($token != null && $token != "") {
            $data = JwtHelper::decode($token);
            if ($data != null) {
                $user_id = $data[LuticateUsersBusiness::KEY_USER_ID];
                $salt = $data[LuticateUsersBusiness::KEY_SALT];
                $user = LuticateUsersBusiness::getById($user_id);
                if ($user->getSalt() != $salt) {
                    $user = null;
                }
            }
        }
        if (is_null($user)) {
            if ($token != null && $token != "") {
                return "Invalid token";
            }
            $user = LuticateUsersBusiness::getById(0);
        }

        self::$_currentUser = $user;
        LuController::$parameters["user"] = $user;

        $permissions[] = LuticatePermissions::USER_LOGIN;
        foreach ($permissions as $permission) {
            if (!LuticatePermissionsBusiness::getUserPermission(self::$_currentUser->getId(), $permission)) {
                return "Permission denied";
            }
        }
        return null;
    }

    /**
     * @param $route LuRoute
     */
    public static function setupAuth($route)
    {
        $route->setMiddleware(function($permissions, $request)
        {
            $auth = self::authFilter($permissions, $request);
            if (is_null($auth)) {
                return true;
            }
            abort(401, $auth);
            return false;
        });
    }

    /**
     * @param $route LuRoute
     */
    public static function setupRoutes($route)
    {
        $prefix = "/luticate";
        $ns = 'Luticate\Auth\Business\\';
        $int = "[0-9]+";
        $group_id = "{group_id:$int}";
        $user_id = "{user_id:$int}";

        $route->post("$prefix/users/login", "${ns}LuticateUsersBusiness", "login");
        $route->post("$prefix/users/logout", "${ns}LuticateUsersBusiness", "logout");
        $route->post("$prefix/users/add", "${ns}LuticateUsersBusiness", "add", LuticatePermissions::USER_ADD);
        $route->post("$prefix/users/$user_id/del", "${ns}LuticateUsersBusiness", "del", LuticatePermissions::USER_DEL);
        $route->post("$prefix/users/$user_id/edit", "${ns}LuticateUsersBusiness", "edit", LuticatePermissions::USER_EDIT);
        $route->post("$prefix/users/me/edit", "${ns}LuticateUsersBusiness", "editMe", LuticatePermissions::USER_EDIT_ME);
        $route->post("$prefix/users/$user_id/setPassword", "${ns}LuticateUsersBusiness", "setPassword", LuticatePermissions::USER_SET_PASSWORD);
        $route->post("$prefix/users/me/setPassword", "${ns}LuticateUsersBusiness", "setPasswordMe", LuticatePermissions::USER_SET_PASSWORD_ME);


        $route->post("$prefix/groups/add", "${ns}LuticateGroupsBusiness", "add", LuticatePermissions::GROUP_ADD);
        $route->post("$prefix/groups/$group_id/del", "${ns}LuticateGroupsBusiness", "del", LuticatePermissions::GROUP_DEL);
        $route->post("$prefix/groups/$group_id/edit", "${ns}LuticateGroupsBusiness", "edit", LuticatePermissions::GROUP_EDIT);
        $route->post("$prefix/groups/$group_id/addUser/$user_id", "${ns}LuticateGroupsBusiness", "addUser", LuticatePermissions::GROUP_ADD_USER);
        $route->post("$prefix/groups/$group_id/delUser/$user_id", "${ns}LuticateGroupsBusiness", "delUser", LuticatePermissions::GROUP_DEL_USER);
    }
}